define(['jquery', 'underscore', 'twigjs'], function ($, _, Twig) {
    var CustomWidget = function () {
        var self = this;
        this.crm_id = 'amocrm';
        this.ENV_DEVELOPMENT = 'development';
        this.ENV_STAGING = 'staging';
        this.ENV_PRODUCTION = 'production';
        this.CURRENT_ENV = self.ENV_PRODUCTION;

        this.bnovobook_domains = [];
        this.bnovobook_domains[self.ENV_STAGING] = 'staging.reservationsteps.ru';
        this.bnovobook_domains[self.ENV_PRODUCTION] = 'reservationsteps.ru';

        this.pms_domains = [];
        this.pms_domains[self.ENV_DEVELOPMENT] = 'bnovopms.booking-desk.ru.loc:4430';
        this.pms_domains[self.ENV_STAGING] = 'staging.pms.bnovo.ru';
        this.pms_domains[self.ENV_PRODUCTION] = 'online.bnovo.ru';

        this.get_bnovobook_domain = function () {
            return self.bnovobook_domains[self.CURRENT_ENV];
        };

        this.get_pms_domain = function () {
            return self.pms_domains[self.CURRENT_ENV];
        };

        //Сбор информации из карточки контакта
        this.start_redirect_to_bnovo_pms = function () {
            let deal_id = AMOCRM.data.current_card.id;
            if (self.system().area == 'lcard' && deal_id != 0) {
                self.get_deal(deal_id);
            } else {
                return false;
            }
        };

        this.get_deal = function (deal_id) {
            let user_id = self.system().amouser_id;
            let token = self.system().amohash;
            let domain = self.system().domain;

            let url = `https://${domain}/api/v2/leads`;
            $.getJSON(url, {
                id: deal_id,
                amouser: user_id,
                amohash: token,
                format: "json"
            }).done(function (data) {
                if (typeof data._embedded.items[0] != "undefined") {
                    let deal = data._embedded.items[0];
                    if (deal.main_contact.id) {
                        self.get_contact(deal.main_contact.id, deal_id);
                    } else {
                        let redirect_params = [];
                        redirect_params['crm_deal_id'] = deal_id;
                        redirect_params['provider_id'] = self.crm_id;
                        redirect_params['provider_object_id'] = domain;
                        redirect_params['open_add_booking_form'] = 1;
                        self.redirect_to_pms(redirect_params);
                    }
                } else {
                    console.log("error, deal is empty")
                }
            }).fail(function() {
                console.log( "error, can't get deal" );
            });
        };

        this.get_contact = function (contact_id, deal_id) {
            let user_id = self.system().amouser_id;
            let token = self.system().amohash;
            let domain = self.system().domain;

            let url = `https://${domain}/api/v2/contacts`;
            $.getJSON(url, {
                id: contact_id,
                amouser: user_id,
                amohash: token,
                format: "json"
            }).done(function (data) {
                if (typeof data._embedded.items[0] != "undefined") {
                    let contact = data._embedded.items[0];
                    let redirect_params = [];
                    redirect_params['provider_id'] = self.crm_id;
                    redirect_params['provider_object_id'] = domain;
                    redirect_params['crm_deal_id'] = deal_id;
                    redirect_params['name'] = contact.name;
                    if (contact.first_name) {
                        redirect_params['name'] = contact.first_name;
                    }
                    redirect_params['surname'] = contact.last_name;
                    redirect_params['crm_contact_id'] = contact.id;
                    redirect_params['open_add_booking_form'] = 1;
                    if (Array.isArray(contact.custom_fields)) {
                        contact.custom_fields.forEach(function (element) {
                            if (element.code == 'EMAIL') {
                                redirect_params['email'] = element.values[0].value
                            }

                            if (element.code == 'PHONE') {
                                redirect_params['phone'] = element.values[0].value
                            }
                        });
                    }
                    self.redirect_to_pms(redirect_params)
                } else {
                    console.log( "contact is empty" );
                }
            }).fail(function() {
                console.log( "error, can't get contact" );
            });
        };

        this.redirect_to_bnovobook = function (params) {
            let bnovobook_link = `https://${self.get_bnovobook_domain()}/rooms/index/` + self.get_settings().uid;
            let redirect_params = $.param(Object.assign({}, params));
            if (redirect_params) {
                bnovobook_link += '?' + redirect_params;
            }
            window.open(bnovobook_link, '_blank');
        };

        this.redirect_to_pms = function (params) {
            console.log(self.get_pms_domain());
            let create_booking_link = `https://${self.get_pms_domain()}/dashboard/`;
            let redirect_params = $.param(Object.assign({}, params));
            if (redirect_params) {
                create_booking_link += '?' + redirect_params;
            }

            console.log(self.get_pms_domain());

            let pms_select_hotel_link = `https://${self.get_pms_domain()}/profile/choose_hotel/${self.get_settings().pms_hotel_id}?return_url=` + btoa(create_booking_link);
            window.open(pms_select_hotel_link, '_blank');
        };

        this.getTemplate = _.bind(function (template, params, callback) {
            params = (typeof params == 'object') ? params : {};
            template = template || '';

            return this.render({
                href: '/templates/' + template + '.twig',
                base_path: this.params.path,
                v: this.get_version(),
                load: callback
            }, params);
        }, this);

        this.callbacks = {
            render: function () {
                if (typeof (AMOCRM.data.current_card) != 'undefined') {
                    // не рендерить на contacts/add || leads/add
                    if (AMOCRM.data.current_card.id == 0) {
                        return false;
                    }
                }

                let create_booking_button_text = self.get_settings().create_booking_button_text;
                let button_text = 'Создать бронь';
                if (create_booking_button_text != 'undefined' && create_booking_button_text.trim() != '') {
                    button_text = create_booking_button_text;
                }

                let widget_html = self.render(
                    {ref: '/tmpl/controls/button.twig'}, // объект data в данном случае содержит только ссылку на шаблон
                    {
                        text: button_text,      //данные
                        class_name: 'bnovobook_button'
                    });

                $('.card-entity-form__main-fields').prepend(widget_html);
                return true;
            },

            init: _.bind(function () {
                console.log('init');
                // AMOCRM.addNotificationCallback(self.get_settings().widget_code, function (data) {
                //     console.log(data)
                // });

                this.add_action("phone", function (params) {
                    /**
                     * код взаимодействия с виджетом телефонии
                     */
                    console.log(params)
                });

                this.add_source("sms", function (params) {
                    /**
                     params - это объект в котором будут  необходимые параметры для отправки смс

                     {
             "phone": 75555555555,   // телефон получателя
             "message": "sms text",  // сообщение для отправки
             "contact_id": 12345     // идентификатор контакта, к которому привязан номер телефона
          }
                     */

                    // return new Promise(_.bind(function (resolve, reject) {
                    //         // тут будет описываться логика для отправки смс
                    //         self.crm_post(
                    //             'https://example.com/',
                    //             params,
                    //             function (msg) {
                    //                 console.log(msg);
                    //                 resolve();
                    //             },
                    //             'text'
                    //         );
                    //     }, this)
                    // );
                });

                return true;
            }, this),
            bind_actions: function () {
                console.log('bind_actions');
                if (self.system().area == 'lcard') {
                    $('.bnovobook_button').on('click', function () {
                        self.start_redirect_to_bnovo_pms();
                    });
                }
                return true;
            },
            settings: function () {
                return true;
            },
            onSave: function () {
                return true;
            },
            destroy: function () {
            },
            contacts: {
                //select contacts in list and clicked on widget name
                selected: function () {
                    console.log('contacts');
                }
            },
            leads: {
                //select leads in list and clicked on widget name
                selected: function () {
                    console.log('leads');
                }
            },
            tasks: {
                //select taks in list and clicked on widget name
                selected: function () {
                    console.log('tasks');
                }
            },
            advancedSettings: _.bind(function () {
                // var $work_area = $('#work-area-' + self.get_settings().widget_code),
                //     $save_button = $(
                //         Twig({ref: '/tmpl/controls/button.twig'}).render({
                //             text: 'Сохранить',
                //             class_name: 'button-input_blue button-input-disabled js-button-save-' + self.get_settings().widget_code,
                //             additional_data: ''
                //         })
                //     ),
                //     $cancel_button = $(
                //         Twig({ref: '/tmpl/controls/cancel_button.twig'}).render({
                //             text: 'Отмена',
                //             class_name: 'button-input-disabled js-button-cancel-' + self.get_settings().widget_code,
                //             additional_data: ''
                //         })
                //     );
                //
                // console.log('advancedSettings');
                //
                // $save_button.prop('disabled', true);
                // $('.content__top__preset').css({float: 'left'});
                //
                // $('.list__body-right__top').css({display: 'block'})
                //     .append('<div class="list__body-right__top__buttons"></div>');
                // $('.list__body-right__top__buttons').css({float: 'right'})
                //     .append($cancel_button)
                //     .append($save_button);
                //
                // self.getTemplate('advanced_settings', {}, function (template) {
                //     var $page = $(
                //         template.render({
                //             title: self.i18n('advanced').title,
                //             widget_code: self.get_settings().widget_code
                //         })
                //     );
                //
                //     $work_area.append($page);
                // });
            }, self),

            /**
             * Метод срабатывает, когда пользователь в конструкторе Salesbot размещает один из хендлеров виджета.
             * Мы должны вернуть JSON код salesbot'а
             *
             * @param handler_code - Код хендлера, который мы предоставляем. Описан в manifest.json, в примере равен handler_code
             * @param params - Передаются настройки виджета. Формат такой:
             * {
             *   button_title: "TEST",
             *   button_caption: "TEST",
             *   text: "{{lead.cf.10929}}",
             *   number: "{{lead.price}}",
             *   url: "{{contact.cf.10368}}"
             * }
             *
             * @return {{}}
             */
            onSalesbotDesignerSave: function (handler_code, params) {
                // var salesbot_source = {
                //         question: [],
                //         require: []
                //     },
                //     button_caption = params.button_caption || "",
                //     button_title = params.button_title || "",
                //     text = params.text || "",
                //     number = params.number || 0,
                //     handler_template = {
                //         handler: "show",
                //         params: {
                //             type: "buttons",
                //             value: text + ' ' + number,
                //             buttons: [
                //                 button_title + ' ' + button_caption,
                //             ]
                //         }
                //     };
                //
                // console.log(params);
                //
                // salesbot_source.question.push(handler_template);
                //
                // return JSON.stringify([salesbot_source]);
            },
        };
        return this;
    };

    return CustomWidget;
});